@@ -8,6 +8,7 @@ import collections
 import random
 import urllib
 import urllib2
+import cookielib
 import urlparse
 import StringIO
 import time
@@ -207,7 +208,7 @@ class Download:
         # attempt downloading content at URL
         while settings.num_retries >= 0 and html is None:
             settings.num_retries -= 1
-            self.proxy = settings.proxy or self.get_proxy()
+            self.proxy = self.get_proxy(settings.proxies)
             # crawl slowly for each domain to reduce risk of being blocked
             self.throttle(url, delay=settings.delay, proxy=self.proxy) 
             html = self.fetch(url, headers=settings.headers, data=settings.data, proxy=self.proxy, user_agent=settings.user_agent, opener=settings.opener, pattern=settings.pattern)
@@ -232,7 +233,7 @@ class Download:
         if html:
             if settings.num_redirects > 0:
                 # allowed to redirect
-                redirect_url = self.get_redirect(url=url, html=html)
+                redirect_url = get_redirect(url=url, html=html)
                 if redirect_url:
                     # found a redirection
                     common.logger.debug('%s redirecting to %s' % (url, redirect_url))
@@ -240,7 +241,10 @@ class Download:
                     html = self.get(redirect_url, **settings) or ''
                     # make relative links absolute so will still work after redirect
                     relative_re = re.compile('(<\s*a[^>]+href\s*=\s*["\']?)(?!http)([^"\'>]+)', re.IGNORECASE)
-                    html = relative_re.sub(lambda m: m.group(1) + urlparse.urljoin(url, m.group(2)), html)
+                    try:
+                        html = relative_re.sub(lambda m: m.group(1) + urlparse.urljoin(url, m.group(2)), html)
+                    except UnicodeDecodeError:
+                        pass
             html = self._clean_content(html=html, max_size=settings.max_size, force_html=settings.force_html, force_ascii=settings.force_ascii)
 
         if self.cache and settings.write_cache:
@@ -312,18 +316,12 @@ class Download:
         return html
 
 
-    def get_redirect(self, url, html):
-        """Check for meta redirects and return redirect URL if found
-        """
-        match = re.compile('<meta[^>]*?url=(.*?)["\']', re.IGNORECASE).search(html)
-        if match:
-            return urlparse.urljoin(url, common.unescape(match.groups()[0].strip())) 
-
-
-    def get_proxy(self):
+    def get_proxy(self, proxies=None):
         """Return random proxy if available
         """
-        if self.settings.proxies:
+        if proxies:
+            proxy = random.choice(proxies)
+        elif self.settings.proxies:
             # select next available proxy
             proxy = random.choice(self.settings.proxies)
         else:
@@ -341,7 +339,7 @@ class Download:
             user_agent = Download.proxy_agents[proxy]
         else:
             # assign random user agent to this proxy
-            user_agent = random.choice(settings.user_agents)
+            user_agent = alg.rand_agent()
             Download.proxy_agents[proxy] = user_agent
         return user_agent
 
@@ -350,9 +348,10 @@ class Download:
         """Simply download the url and return the content
         """
         self.error_content = None
-        common.logger.info('Downloading %s' % url)
         # create opener with headers
-        opener = opener or urllib2.build_opener()
+        if not opener:
+            cj = cookielib.CookieJar()
+            opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
         if proxy:
             if url.lower().startswith('https://'):
                 opener.add_handler(urllib2.ProxyHandler({'https' : proxy}))
@@ -360,16 +359,18 @@ class Download:
                 opener.add_handler(urllib2.ProxyHandler({'http' : proxy}))
         
         headers = headers or {}
-        for k, v in settings.default_headers.items():
-            if k not in headers:
-                if k == 'Referer':
-                    v = url
-                headers[k] = v
         headers['User-agent'] = user_agent or self.get_user_agent(proxy)
+        headers['Accept-encoding'] = 'gzip'
+        for name, value in settings.default_headers.items():
+            if name not in headers:
+                if name == 'Referer':
+                    value = url
+                headers[name] = value
         
         if isinstance(data, dict):
             # encode data for POST
             data = urllib.urlencode(data) 
+        common.logger.info('Downloading %s %s' % (url, data or ''))
 
         try:
             request = urllib2.Request(url, data, headers)
@@ -396,10 +397,10 @@ class Download:
                     self.error_content = ''
             # so many kinds of errors are possible here so just catch them all
             common.logger.warning('Download error: %s %s' % (url, e))
-            if not self.settings.acceptable_errors or self.response_code not in self.settings.acceptable_errors:
-                content, self.final_url = None, url
-            else:
+            if self.settings.acceptable_errors and self.response_code in self.settings.acceptable_errors:
                 content, self.final_url = self.settings.default, url
+            else:
+                content, self.final_url = None, url
         return content
 
 
@@ -440,7 +441,7 @@ class Download:
                     common.logger.debug('Reloaded proxies from updated file.')
 
 
-    def geocode(self, address, delay=5, read_cache=True, num_retries=1):
+    def geocode(self, address, delay=5, read_cache=True, num_retries=1, language=None):
         """Geocode address using Google's API and return dictionary of useful fields
 
         address:
@@ -449,8 +450,12 @@ class Download:
             how long to delay between API requests
         read_cache:
             whether to load content from cache when exists
+        num_retries:
+            the number of times to try downloading
+        language:
+            the language to set
         """
-        url = 'http://maps.google.com/maps/api/geocode/json?address=%s&sensor=false' % urllib.quote_plus(address)
+        url = 'http://maps.google.com/maps/api/geocode/json?address=%s&sensor=false%s' % (urllib.quote_plus(address), '&language=' + language if language else '')
         html = self.get(url, delay=delay, read_cache=read_cache, num_retries=num_retries)
         results = collections.defaultdict(str)
         if html:
@@ -665,6 +670,14 @@ class Download:
             return save_path
 
 
+def get_redirect(url, html):
+    """Check for meta redirects and return redirect URL if found
+    """
+    match = re.compile('<meta[^>]*?url=(.*?)["\']', re.IGNORECASE).search(html)
+    if match:
+        return urlparse.urljoin(url, common.unescape(match.groups()[0].strip())) 
+
+
 
 def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=None, wait_finish=True, reuse_queue=False, max_queue=1000, **kwargs):
     """Download these urls in parallel
@@ -729,7 +742,7 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
                         if cb:
                             try:
                                 # use callback to process downloaded HTML
-                                cb_urls = cb(D, url, html)
+                                result = cb(D, url, html)
 
                             except Exception, e:
                                 # catch any callback error to avoid losing thread
@@ -737,9 +750,10 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
 
                             else:
                                 # add these URL's to crawl queue
-                                for cb_url in cb_urls or []:
-                                    if isinstance(cb_urls, dict):
-                                        DownloadThread.discovered[cb_url] = cb_urls[cb_url]
+                                for link in result or []:
+                                    cb_url = urlparse.urljoin(url, link)
+                                    if isinstance(result, dict):
+                                        DownloadThread.discovered[cb_url] = result[link]
                                     else:
                                         DownloadThread.discovered[cb_url] = DEFAULT_PRIORITY
                                             
@@ -875,7 +889,7 @@ class State:
         fp.close()
         try:
             if os.name == 'nt': 
-                # on wineows can not rename if file exists
+                # on windows can not rename if file exists
                 if os.path.exists(self.output_file):
                     os.remove(self.output_file)
             # atomic copy to new location so state file is never partially written
