@@ -22,6 +22,7 @@ from PySide.QtCore import QByteArray, QUrl, QTimer, QEventLoop, QIODevice, QObje
 from PySide.QtWebKit import QWebFrame, QWebView, QWebPage, QWebSettings
 from PySide.QtNetwork import QNetworkAccessManager, QNetworkProxy, QNetworkRequest, QNetworkReply, QNetworkDiskCache
 """
+import alg
 import common
 import settings
 
@@ -302,10 +303,9 @@ class WebkitBrowser(QWebView):
     def __init__(self, base_url=None, gui=False, user_agent=None, proxy=None, load_images=False, forbidden_extensions=None, allowed_regex='.*?', timeout=20, delay=5, enable_plugins=False):
         self.app = QApplication(sys.argv) # must instantiate first
         QWebView.__init__(self)
-        #super(WebkitBrowser, self).__init__()
         manager = NetworkAccessManager(proxy, forbidden_extensions, allowed_regex)
         manager.finished.connect(self.finished)
-        webpage = WebPage(user_agent or random.choice(settings.user_agents))
+        webpage = WebPage(user_agent or alg.rand_agent())
         webpage.setNetworkAccessManager(manager)
         self.setPage(webpage)
         self.setHtml('<html><head></head><body>No content loaded</body></html>', QUrl('http://localhost'))
@@ -313,7 +313,7 @@ class WebkitBrowser(QWebView):
         self.delay = delay
         self.base_url = base_url
         self.jquery_lib = None
-        #enable flash plugin etc.
+        # enable flash plugin etc.
         self.settings().setAttribute(QWebSettings.PluginsEnabled, enable_plugins)
         self.settings().setAttribute(QWebSettings.JavaEnabled, enable_plugins)
         self.settings().setAttribute(QWebSettings.AutoLoadImages, load_images)
